* The purpose of this SAS program is to demonstrate that the SCORE-CHI-SQUARE
from PROC LOGISTIC MODEL Y = <> /  SELECTION=SCORE can be obtained without
ever maximizing the Likelihood fuction. In the PROC LOGISTIC that is
run in the middle of this program the MAXITER is set to 0. There are no iterations
performed in solving the likelihood fucntion. The purpose of this PROC LOGISTIC 
is to produce the Covariance matrix. This is a factor that is needed in the
calculation of the SCORE-CHI-SQUARE.

At the end of this program there is a real PROC LOGISTIC that actually runs the
SELCTION = SCORE. Compare these results which the results that are produced by 
the preceeding DATA step.
;

* STEP 0: Create example for PROC LOGISTIC;
data work;
	do i = 1 to 1000;
		w1 = 0.10*rannor(3);
		w2 = 0.15*rannor(3);
		y = (ranuni(1) < .5 + 0.1*w1 + .005*w2);
		output;
		end;
run;
* STEP 1: Obtain Intercept for Logistic Model with no predictors;
* First find the mean of Y;
proc means data = work noprint; var y;
output out = meanout(keep= meanY)
mean = meanY;
run;
* Now transform meanY to the Logistic Intercept (no predictors);
DATA Estimate; SET meanout;
intercept = log(meanY/(1 - meanY));
run;

* STEP 2: Compute Score statistics for w1 and w2;
* The Score statistics are conditional on the Intercept;
* The Score equations (below) are the first derivatives of the Log-Likelihood function but with the Intercept "in the model";
DATA U; SET Estimate work end=eof;

RETAIN b0 Uw1 Uw2 0;
KEEP Uw1 Uw2 _LINK_;

IF _N_ = 1 then do;
b0 = intercept;
put b0 = ;
end;

IF _N_ > 1
THEN
DO;
* Score Statistics;
	Uw1 = Uw1 + y*w1 - w1*(exp(b0) / (1 + exp(b0))); 
	Uw2 = Uw2 + y*w2 - w2*(exp(b0) / (1 + exp(b0))); 
	END;

if eof then do;
length _LINK_ $8;
_LINK_ = "LOGIT"; /* Used for MERGE below */
	OUTPUT;
	end;	
run;

/* For use in initializing coefficients in PROC LOGISTIC */
data initial; set Estimate;
	w1=0; w2=0;
run;
/* Suppress unneeded Logistic output */
ods select ALL;
ods exclude EffectNotInModel;
ods exclude Association;
ods exclude ClassLevelInfo;
ods exclude Classification;
ods exclude ConvergenceStatus;
ods exclude FitStatistics;
ods exclude GlobalTests;
ods exclude OddsRatios; 
ods exclude ParameterEstimates;
ods exclude ResidualChiSq; 
ods exclude Type3;
/* MAXITER = 0 to obtain Covariance Matrix with evaluation at Initial Values of Parameters */
proc logistic data = work descending outest = est_w covout INEST= initial;
model y = w1 w2  / maxiter = 0;
run;
* STEP 3: Compute Score Chi-Square for w1 and w2;
data score_chi_sq; merge U est_w(where = (_name_ in ("w1", "w2"))) END=EOF; by _LINK_;
length VariablesInModel $5000;
retain score_chi_sq_sum 0;
if _name_ = "w1" then do;
	score_chi_sq = w1*Uw1**2;
	VariablesInModel = "w1";
	output;
	score_chi_sq_sum = w1*Uw1**2 + w2*Uw1*Uw2 ;
	end;
if _name_ = "w2" then do;
	score_chi_sq = w2*Uw2**2;
	VariablesInModel = "w2";
	output;
	score_chi_sq_sum = score_chi_sq_sum + w1*Uw1*Uw2 + w2*Uw2**2;
	end;
IF EOF THEN DO;
	score_chi_sq = score_chi_sq_sum;
	VariablesInModel = "w1 w2";
	output;
	END;
run;


* COMPARE Calculations above to SELECTION=SCORE ;
proc logistic data = work desc;
model y = w1 w2 / selection = score;
run;

proc print data = score_chi_sq; var score_chi_sq VariablesInModel;
run;
